'''犯人'''
from ..model import on_cmd, Game, Player
from ..config import R


@on_cmd(cmds=R.犯人, states="game", max_num=1, auto_turn_next=False)
async def criminal(game: Game, player: Player):
    player.is_good = False
    await game.send(f"{R.犯人}是 {player.index_name}！")

    # 被警部抓到了
    if game.police.target_id == player.id:
        p2 = game.get_player(game.police.owner_id)
        p2.is_good = True
        await game.send(f"但是{R.犯人}被 {p2.index_name} 布置的{R.警部}抓到了！")
        return await game.end(True)

    await game.end(False)



'''游戏局势'''
from .cat import cat, get_uid
from .model import Game
from .config import R


@cat.on_cmd(cmds="局势", states="*")
async def show_game_info():
    '''查看场面局势'''
    game = cat.get_data(Game)
    if not game.start:
        return

    items = ["所有玩家剩余手牌数"]
    for p in game.players:
        items.append(f"{p.index_name} {len(p.cards)}张")

    # 统计牌数
    detect_num = 0
    cert_num = 0
    for p in game.players:
        for card in p.cards:
            if card == R.侦探:
                detect_num += 1
            elif card == R.不在场证明:
                cert_num += 1

    if detect_num:
        items.append(f"剩余{R.侦探}牌 {detect_num}张")

    if cert_num:
        items.append(f"剩余{R.不在场证明}牌 {cert_num}张")

    items.append(f"目前轮到 {game.current_player.index_name}")

    await cat.send("\n".join(items))


@cat.on_cmd(cmds="手牌", states="*")
async def show_cards():
    '''展示手牌'''
    game = cat.get_data(Game)
    if not game.start:
        return
    
    uid = get_uid()
    
    player = game.get_player(uid)
    if not player:
        return

    items = ["您的手牌\n", *player.cards]
    await player.send("\n".join(items))



'''所有模型，也是本插件耦合度最高的地方（狮 山 核 心）'''
import inspect
import asyncio
from random import choice
from typing import Awaitable, Callable, Optional
from pydantic import BaseModel
from .cat import cat, get_uid, help_dict
from .config import R, config


class User(BaseModel):
    '''房间成员'''
    id: str
    name: str


class Room(BaseModel):
    '''房间'''
    users: list[User] = []
    cards: list[str] = []

    @property
    def info(self):
        items = ["当前房间人员\n"]
        items.extend(u.name for u in self.users)
        return "\n".join(items)

    def join(self, id: str, name: str):
        for u in self.users:
            if u.id == id:
                return
        else:
            user = User(id=id, name=name)
            self.users.append(user)
            return True

    def leave(self, id: str):
        for u in self.users:
            if u.id == id:
                self.users.remove(u)
                return True


class Player(BaseModel):
    '''玩家'''
    index: int

    id: str
    name: str

    cards: list[str] = []
    '''当前手牌'''

    is_good: bool = True
    '''天生都是好人，打出共犯或犯人后变坏'''

    game: "Game"

    fut: Optional[asyncio.Future]
    '''超时控制'''

    choose_card: Optional[Callable[[str], Awaitable]]
    '''需要执行的任务（放一张卡牌）'''

    class Config:
        arbitrary_types_allowed = True

    @property
    def index_name(self):
        '''座号+名字'''
        return f"[座号{self.index+1}] {self.name}"

    async def send(self, msg: str):
        '''发送私聊消息'''
        await cat.send_private(self.id, msg)

    async def check(self, card: str, max_num: int = 4, at_require: bool = False):
        '''检查牌是否可以打出'''
        if card != R.第一发现人 and not self.game.first:
            await self.game.send("第一张牌必须是第一发现人")
            return False
        if self.game.current_player.id != self.id:
            await self.game.send("没轮到您")
            return False
        if card not in self.cards:
            await self.game.send(f"{self.index_name} 没有{card}")
            items = ["你没有这张牌，你当前的手牌是\n", *self.cards]
            await self.send("\n".join(items))
            return False
        if len(self.cards) > max_num:
            await self.game.send(f"{card}只能在手牌<={max_num}时打出")
            return False
        if at_require and not self.game.get_player(cat.event.at):
            await self.game.send("您需要at一个游戏中的玩家")
            return False
        return True

    async def play_card(self, card: str):
        '''打出一张牌并通知'''
        self.cards.remove(card)
        await self.game.send(f"{self.index_name} 打出{card}")
        if config.auto_card_help:
            await self.game.send(help_dict[card])

        # 结束超时任务
        if self.fut and not self.fut.done():
            self.fut.set_result(True)


class MarkItem(BaseModel):
    '''标志物'''
    target_id: str = ""
    '''目标id'''
    owner_id: str = ""
    '''主人id'''


class GiveAction(BaseModel):
    giver: Optional[Player]
    receiver: Optional[Player]
    card: Optional[str]

    @property
    def ready(self):
        return self.giver and self.receiver and self.card

    def convey(self):
        if self.ready:
            self.giver.cards.remove(self.card)
            self.receiver.cards.append(self.card)


class RoundGive(BaseModel):
    gives: list[GiveAction] = []

    def init(self, givers: list[Player]):
        self.gives = [GiveAction(giver=p) for p in givers]

    def get_give(self, giver_id: str):
        for g in self.gives:
            if g.giver.id == giver_id:
                return g

    @property
    def all_given(self):
        '''所有人给牌完毕'''
        for g in self.gives:
            if not g.card:
                return False
        return True

    def set_receivers(self):
        '''手牌给上家'''
        gs = [self.gives[-1], *self.gives[:-1]]
        rs = [g.giver for g in gs]
        for r, g in zip(rs, self.gives):
            g.receiver = r

    async def convey_all(self):
        for g in self.gives:
            g.convey()
            await g.giver.send(f"{g.receiver.index_name} 获得了您的{g.card}")

        for g in self.gives:
            await g.receiver.send(f"您获得了 {g.giver.index_name} 的{g.card}")
            items = ["您当前的手牌\n", *g.receiver.cards]
            await g.receiver.send("\n".join(items))


class Game(BaseModel):
    '''游戏'''
    start: bool = False

    players: list[Player] = []
    '''玩家列表'''
    index: int = 0
    '''玩家指针'''
    first: bool = False
    '''第一发现人是否已打出'''

    dog: MarkItem = MarkItem()
    '''神犬标志物'''
    police: MarkItem = MarkItem()
    '''警部标志物'''

    lock: asyncio.Lock = asyncio.Lock()
    '''同步锁，保证用户操作的原子性，防止一个用户因某种情况连出两牌的情况'''
    group_id: str = ""
    '''群聊号'''

    round_give: RoundGive = RoundGive()
    '''环绕给牌'''

    class Config:
        arbitrary_types_allowed = True

    def init(self, room: Room, group_id: str):
        '''将房间成员转换为游戏玩家'''
        # 保存群号
        self.group_id = group_id

        # 将room成员转换为玩家
        self.players = [
            Player(
                id=u.id, name=u.name,
                index=i, game=self,
                cards=room.cards[i*4:(i+1)*4],
            )
            for i, u in enumerate(room.users)
        ]

        # 设置玩家指针
        for i, p in enumerate(self.players):
            if R.第一发现人 in p.cards:
                self.index = i
                break

        # 第一发现人未打出
        self.first = False

        # 正式开始
        self.start = True

    async def send(self, msg: str):
        '''发送群聊消息'''
        await cat.send_group(self.group_id, msg)

    @property
    def current_player(self):
        return self.players[self.index]

    def set_state(self, state: str):
        cat.state = state

    async def turn_next(self):
        '''转移至下一个有牌的玩家'''
        n = len(self.players)
        for i in range(n):
            self.index = (self.index+1) % n
            if self.current_player.cards:
                break

        await cat.send(f"现在轮到 {self.current_player.index_name} 出牌")
        overtime(self.current_player)

    def get_player(self, uid: str):
        for p in self.players:
            if p.id == uid:
                return p

    async def end(self, good_win: bool):
        '''游戏终结，给出输赢'''
        for p in self.players:
            # 结束超时任务
            if p.fut and not p.fut.done():
                p.fut.cancel()

        goods = []
        bads = []
        for p in self.players:
            if p.is_good:
                goods.append(p.name)
            else:
                bads.append(p.name)

        if good_win:
            winners, losers = goods, bads
        else:
            winners, losers = bads, goods

        items = [
            "，".join(winners) + "赢了",
            "，".join(losers) + "输了",
        ]
        await self.send("\n".join(items))
        await self.send("已返回房间")
        self.set_state("room")

        self.start = False


Player.update_forward_refs()


class AtPlayer(Player):
    '''被at的玩家'''
    pass


def on_cmd(
    cmds: str | list[str] = "",
    states: str | list[str] = "",
    max_num: int = 4,
    at_require: bool = False,
    auto_check_card: bool = True,
    auto_play_card: bool = True,
    auto_turn_next: bool = True
):
    '''
    - 排除私聊发送的消息
    - 排除非游戏玩家
    - 检查第一发现人、行动权、是否有手牌、是否满足数量要求、是否at玩家
    - 打出牌
    - 执行回调（自定义）
    - 轮到下家
    '''
    def decorator(func: Callable[..., Awaitable]):

        # 分析func参数表
        # 参数名:类型
        params = inspect.signature(func).parameters
        params = {k: v.annotation for k, v in params.items()}

        async def _func():
            # 排除私聊发送的消息
            if cat.event.private_forward_id:
                return await cat.send("请在群聊里打牌")

            game = cat.get_data(Game)

            uid = get_uid()

            # 排除未参加游戏的人
            player = game.get_player(uid)
            if not player:
                return

            async with game.lock:
                card = cat.cmd
                if auto_check_card:
                    if not await player.check(card, max_num, at_require):
                        return

                    if auto_play_card:
                        await player.play_card(card)

                # 依赖注入
                _params = {}

                for key, cls in params.items():
                    if cls is Game:
                        _params[key] = game
                    elif cls is Player:
                        _params[key] = player
                    elif cls is AtPlayer:
                        p2 = game.get_player(cat.event.at)
                        _params[key] = p2
                    elif cls is str:
                        _params[key] = card

                await func(**_params)

                if auto_turn_next:
                    await game.turn_next()

        _func.__name__ = func.__name__
        cat.on_cmd(cmds=cmds, states=states, auto_help=False)(_func)
        return func

    return decorator


def overtime_wrapper(func: Callable[[Player], Awaitable]):
    '''超时任务装饰器'''

    async def overtime(player: Player):
        try:
            await asyncio.wait_for(player.fut, config.overtime)
        except asyncio.exceptions.TimeoutError:
            async with player.game.lock:
                await func(player)

    def set_start(player: Player):
        loop = asyncio.get_event_loop()
        player.fut = loop.create_future()
        loop.create_task(overtime(player))

    return set_start


@overtime_wrapper
async def overtime(player: Player):
    '''打牌超时'''
    game = player.game
    card = choice(player.cards)
    player.cards.remove(card)
    await game.send(f"{player.index_name} 被系统强制丢弃了{card}")
    if card == R.犯人:
        player.is_good = False
        await game.end(True)
    else:
        await game.turn_next()


@cat.on_cmd(cmds=["退出游戏", "退出"], states="*")
async def _():
    await cat.rest()
    cat.remove_private_redirect()
    game = cat.get_data(Game)
    for p in game.players:
        # 结束超时任务
        if p.fut and not p.fut.done():
            p.fut.set_result(True)



'''房间'''
from ayaka import get_adapter
from .utils import shuffle, get_cards
from .cat import cat, get_uid
from .config import R
from .model import Room, Game, Player, overtime_wrapper


@cat.on_cmd(cmds=f"{R.犯人}在跳舞")
async def wakeup():
    '''唤醒猫猫'''
    await cat.wakeup("room")
    await cat.send_help()
    room = cat.get_data(Room)
    room.users = []
    await join_room()


@cat.on_cmd(cmds="加入", states="room")
async def join_room():
    '''加入房间'''
    # 排除私聊转发
    if cat.event.private_forward_id:
        return

    name = cat.user.name
    room = cat.get_data(Room)

    try:
        uid = get_uid()
        await cat.send_private(uid, "测试私聊，无需回复")
    except:
        await cat.send(f"{name} 不是bot好友，无法加入游戏")

        if get_adapter().name == "nb2.ob11.qqguild_patch":
            await cat.send("频道用户请使用命令：绑定私聊 <qq号>\nbot将通过qq私聊给您发牌")
    else:
        cat.add_private_redirect(uid)

        room.join(uid, name)
        room.cards = []

        await cat.send(f"{name} 加入房间")
        await cat.send(room.info)


@cat.on_cmd(cmds="离开", states="room")
async def leave_room():
    '''离开房间'''
    # 排除私聊转发
    if cat.event.private_forward_id:
        return

    name = cat.user.name
    room = cat.get_data(Room)

    uid = get_uid()

    room.leave(uid)
    room.cards = []

    cat.remove_private_redirect(uid)

    await cat.send(f"{name} 离开房间")
    await cat.send(room.info)

    if not room.users:
        await cat.send("房间为空，游戏结束")
        await cat.rest()


@cat.on_cmd(cmds="房间", states="room")
async def show_room():
    '''查看房间信息'''
    # 排除私聊发送的消息
    if cat.event.private_forward_id:
        return

    room = cat.get_data(Room)
    await cat.send(room.info)


@cat.on_cmd(cmds="开始", states="room")
async def start_game():
    '''开始游戏'''
    # 排除私聊发送的消息
    if cat.event.private_forward_id:
        return

    room = cat.get_data(Room)

    # 人数控制
    n = len(room.users)
    if n < 3 or n > 8:
        return await cat.send(f"{R.犯人}在跳舞只能3-8人游玩，现在房间里有{n}人")

    # 生成牌库
    if not room.cards:
        room.cards = get_cards(n)

    # 洗牌
    shuffle(room.cards)

    # 状态转移
    cat.state = "game"

    # 初始化游戏
    game = cat.get_data(Game)
    game.init(room, cat.session.id)

    # 通知
    items = ["游戏开始\n"]
    items.extend(p.index_name for p in game.players)
    await game.send("\n".join(items))

    # 发牌
    for player in game.players:
        items = ["您的手牌是\n", *player.cards]
        await player.send("\n".join(items))

    # 第一发现人
    await game.send(f"{R.第一发现人}是 {game.current_player.index_name}")

    # 设置超时任务
    overtime(game.current_player)


@overtime_wrapper
async def overtime(player: Player):
    card = R.第一发现人
    player.cards.remove(card)
    player.game.first = True
    await player.game.send(f"{player.index_name} 被系统强制丢弃了{card}")
    await player.game.turn_next()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['criminal_dance', 'criminal_dance.cards']

package_data = \
{'': ['*']}

install_requires = \
['ayaka>=0.0.4.0,<0.0.5.0']

setup_kwargs = {
    'name': 'criminal-dance',
    'version': '0.2.0',
    'description': '犯人在跳舞',
    'long_description': '<div align="center">\n\n# 犯人在跳舞 0.2.0\n\n祝各位新年快乐~\n\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/criminal_dance)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/criminal_dance)\n![PyPI - License](https://img.shields.io/pypi/l/criminal_dance)\n![PyPI](https://img.shields.io/pypi/v/criminal_dance)\n\n基于ayaka的文字版桌游！\n\n</div>\n\n得益于[ayaka](https://github.com/bridgeL/ayaka)，本插件可作为如下机器人框架的插件使用\n\n- [nonebot2](https://github.com/nonebot/nonebot2)(使用[onebot11](https://github.com/nonebot/adapter-onebot)适配器，可兼容qq频道)\n- [hoshino](https://github.com/Ice-Cirno/HoshinoBot)\n- [nonebot1](https://github.com/nonebot/nonebot)\n\n也可将其作为console程序离线运行\n\n## nonebot2 安装本插件\n\n```\npip install criminal_dance\n```\n\n## hoshino 安装本插件\n\n- 从github仓库下载\n- 安装依赖`pip install -r requirements.txt`\n- 将`criminal_dance/criminal_dance`文件夹放入`hoshino/modules`\n- 在`hoshino/config/__bot__.py`中的`MODULES_ON`里，写入字符串`criminal_dance`\n\n## 作为console程序离线运行\n\n```py\n# run.py\nimport ayaka.adapters.console as cat\nimport criminal_dance\nif __name__ == "__main__":\n    cat.run()\n```\n\n```\npython run.py\n```\n\n## 帮助\n\n```\n[犯人在跳舞]\n3-8人游玩，游玩前请先加bot好友，否则无法发牌。90s内不出牌会被系统强制弃牌（防止挂机）\n- 犯人在跳舞 唤醒猫猫\n[*]\n- 卡牌帮助 <卡牌名> 获取相应的帮助\n- 详细帮助 \n- 牌库规则 \n- 绑定私聊 <qq uid> 频道用户请使用该命令\n- 退出游戏 \n- 局势 查看场面局势\n- 手牌 展示手牌\n[room]\n- 加入 加入房间\n- 离开 离开房间\n- 房间 查看房间信息\n- 开始 开始游戏\n```\n\n### 使用流程\n\n所有参与者都要提前加bot的qq好友，以便bot通过私聊发消息给你们\n\n- 发送`犯人在跳舞`，启动猫猫并创建游戏房间，发起者自动位于1号位\n- 频道用户需要额外发送`绑定私聊 12345678`，绑定自己的qq号\n- 随后其他人发送`加入`，根据加入顺序确定座次\n- 人数满足要求后，发送`开始`开始游戏，bot将通过私聊告知手牌\n- 之后在群里发送卡牌名称即可打出此卡！\n- 发送`退出游戏`，关闭猫猫！\n\n### 卡牌帮助\n\n```\n[第一发现人] 一切都是由您开始。打出时没有特别效果\n[共犯] 打出这张牌成为共犯。当犯人获胜时，您也获得胜利。当犯人输掉游戏时，您也跟着输掉游戏\n[犯人] 您是犯人，不能让其他玩家知道。您只能在只剩下这张手牌时才能打出这张牌，如果您要这么做，您获胜\n[不在场证明] 只要有了这个，您就不是犯人了。打出时没有特别效果。如果您有犯人和不在场证明，侦探质疑时您可以否认\n[侦探] 您的手牌<=2才能打出这张牌。打出时质疑另一位玩家，如果该玩家持有犯人，您获胜\n[普通人] 打出时没有特别效果\n[谣言] 所有玩家随机抽一张他下家玩家的手牌\n[情报交换] 所有玩家把一张牌传给他上家玩家\n[目击者] 看另一个玩家的手牌\n[交易] 和至少还有1张手牌的玩家交换一张手牌。如果这是您打出的最后一张牌，则没有特别效果\n[神犬] 选择一个玩家。该玩家弃掉他其中的一张手牌，并且获得神犬。如果他弃掉一张犯人牌，您获胜\n[警部] 手牌<=2时，选定一个玩家放置此牌，若其最终打出犯人牌，您获得游戏胜利\n```\n\n### 详细帮助\n\n```\n开局每人4张手牌，轮流出牌，整局游戏的第一张牌必须是{R.第一发现人}（类似扑克规则中的红桃3）\n\n游戏分为好人阵营和坏人阵营，默认都是好人，而最终打出{R.犯人}牌的人作为坏人而胜利\n但{R.犯人}牌只有在手牌数为1的时候才能打出，此时打出者作为{R.犯人}而胜利\n其他人的目标就是在{R.犯人}逃跑成功之前，通过{R.侦探}、{R.神犬}、{R.警部}等牌抓到{R.犯人}，此时好人阵营胜利\n当你打出{R.共犯}牌后，你便加入了坏人阵营，需要协助{R.犯人}获胜\n当然，你也可以当个二五仔，若{R.共犯}使用{R.侦探}等牌抓到了{R.犯人}，那么他也视为好人阵营一同胜利\n\n根据参与人数的不同，牌库的牌也不同，具体规则请发送 牌库规则 进一步了解\n\n此外，当游戏进行中时，还有如下命令\n局势：获得游戏进行情况等信息\n手牌：获得你当前的手牌情况（bot私聊告知）\n```\n\n### 牌库规则\n\n```\n3人局，必须有第一发现人、犯人、侦探、不在场证明，加其他任意8张牌\n4人局，必须有第一发现人、犯人、侦探、不在场证明、共犯，加其他任意11张牌\n5人局，必须有第一发现人、犯人、侦探、不在场证明*2、共犯，加其他任意14张牌\n6人局，必须有第一发现人、犯人、侦探*2、不在场证明*2、共犯*2，加其他任意16张牌\n7人局，必须有第一发现人、犯人、侦探*2、不在场证明*3、共犯*2，加其他任意19张牌\n8人局，加全部\n```\n\n## 配置\n\n`data/ayaka/犯人在跳舞.json`\n\n| 属性           | 意义                                                                              |\n| -------------- | --------------------------------------------------------------------------------- |\n| overtime       | 超时限制，超时后系统会自动出牌，防止挂机                                          |\n| auto_card_help | 自动卡牌帮助，每打出一张牌后bot都会发送帮助，等参与玩家熟悉规则后，可以关闭该配置 |\n| rename         | 重命名卡牌，为担心和谐的人准备                                                    |\n\n你可以将犯人改名为大老鼠，共犯改成小老鼠，侦探改成猫猫，警部改成捕鼠笼，谣言改成幸运大转盘，例如：\n\n```json\n{\n    "第一发现人": "第一发现人",\n    "犯人": "大老鼠",\n    "神犬": "神犬",\n    "警部": "捕鼠笼",\n    "共犯": "小老鼠",\n    "普通人": "普通人",\n    "不在场证明": "不在场证明",\n    "目击者": "目击者",\n    "侦探": "猫猫",\n    "谣言": "幸运大转盘",\n    "交易": "交易",\n    "情报交换": "情报交换"\n}\n```\n\n此时，游戏的启动命令将变成 `大老鼠在跳舞`\n\n## 更新记录\n\n### 0.0.1b2\n\n- 原型阶段，勉强可玩\n\n### 0.0.1 \n\n- 重构代码，第一个正式版本，全部功能已实现\n\n### 0.0.2 \n\n- 修复BUG：在交易、情报交换等特殊阶段，无法查询手牌、局势\n\n### 0.0.3\n\n- 重构代码，更精简的逻辑结构\n- 修复BUG：在关闭猫猫后依旧可以通过私聊控制群聊的猫猫\n- 更新依赖 ayaka 0.0.1.4\n\n### 0.0.4b0\n\n- 更新退出游戏命令\n- 修复BUG：使用强制退出时无法关闭超时机制，请使用退出游戏命令\n- 更新依赖 ayaka 0.0.1.5b1\n\n### 0.1.0\n\n- 修复BUG：结束游戏后超时机制没有正确关闭\n- 修复BUG：神犬、侦探牌打出后没有按预期切换牌权\n- 更新依赖 ayaka 0.0.2.0\n\n### 0.1.1\n\n- 更新依赖 ayaka 0.0.3.0\n- 新特性：支持qq频道\n\n\n### 0.1.2\n\n- 更严格的依赖约束\n- 新特性：新增命令退出\n- 优化游戏帮助\n\n### 0.2.0\n\n- 更新依赖\n- 修复BUG：修复了侦探指认犯人时，犯人没有被算入游戏失败方的BUG\n',
    'author': 'Su',
    'author_email': 'wxlxy316@163.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/bridgeL/criminal_dance',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



'''侦探'''
from ..model import on_cmd, Game, Player, AtPlayer
from ..config import R


@on_cmd(cmds=R.侦探, states="game", max_num=2, at_require=True, auto_turn_next=False)
async def detect(game: Game, player: Player, p2: AtPlayer):
    await game.send(f"目标是 {p2.index_name}！")

    # 侦探排查中...
    if R.犯人 in p2.cards and R.不在场证明 not in p2.cards:
        player.is_good = True
        p2.is_good = False
        return await game.end(True)

    await game.send(f"{p2.index_name} 不是{R.犯人}~")
    await game.turn_next()



'''神犬'''
from random import choice
from ..model import on_cmd, overtime_wrapper, Game, Player, AtPlayer
from ..config import R


@on_cmd(cmds=R.神犬, states="game", at_require=True, auto_play_card=False, auto_turn_next=False)
async def dog(game: Game, player: Player, p2: AtPlayer, card: str):
    if p2.id == player.id:
        return await game.send(f"{R.神犬}不咬主人")

    if not p2.cards:
        return await game.send(f"{R.神犬}只能扑向有手牌的玩家")

    game.dog.target_id = p2.id
    game.dog.owner_id = player.id

    await player.play_card(card)
    await game.send(f"目标是 {p2.index_name}！请TA丢弃一张牌")

    game.set_state("dog")

    # 设置超时任务
    overtime(p2)


@on_cmd(cmds=[R.共犯, R.普通人, R.不在场证明, R.目击者, R.侦探, R.交易, R.谣言, R.情报交换, R.警部, R.犯人], states="dog", auto_play_card=False, auto_check_card=False, auto_turn_next=False)
async def dog_bite(game: Game, player: Player, card: str):
    if game.dog.target_id != player.id:
        return

    if card not in player.cards:
        return await game.send(f"{player.index_name} 没有{card}")

    await player.play_card(card)

    if card == R.犯人:
        player.is_good = False
        owner = game.get_player(game.dog.owner_id)
        owner.is_good = True
        return await game.end(True)

    player.cards.append(R.神犬)
    await game.send(f"{player.index_name} 获得{R.神犬}牌")

    game.set_state("game")
    await game.turn_next()


@overtime_wrapper
async def overtime(player: Player):
    game = player.game
    card = choice(player.cards)
    player.cards.remove(card)
    await game.send(f"{player.index_name} 被系统强制丢弃了{card}")

    if card == R.犯人:
        player.is_good = False
        owner = game.get_player(game.dog.owner_id)
        owner.is_good = True
        return await game.end(True)

    player.cards.append(R.神犬)
    await game.send(f"{player.index_name} 获得{R.神犬}牌")

    game.set_state("game")
    await game.turn_next()
